@@ -1,6 +1,9 @@
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <dirent.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-#include <X11/Xcursor/Xcursor.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -10,12 +13,18 @@
 #include "gnome-theme-info.h"
 #include "gtkrc-utils.h"
 
+#ifdef HAVE_XCURSOR
+#include <X11/Xcursor/Xcursor.h>
+#endif
+
 #define THEME_NAME "X-GNOME-Metatheme/Name"
 #define THEME_COMMENT "X-GNOME-Metatheme/Comment"
 #define GTK_THEME_KEY "X-GNOME-Metatheme/GtkTheme"
 #define GTK_COLOR_SCHEME_KEY "X-GNOME-Metatheme/GtkColorScheme"
 #define METACITY_THEME_KEY "X-GNOME-Metatheme/MetacityTheme"
 #define ICON_THEME_KEY "X-GNOME-Metatheme/IconTheme"
+#define CURSOR_THEME_KEY "X-GNOME-Metatheme/CursorTheme"
+#define CURSOR_SIZE_KEY "X-GNOME-Metatheme/CursorSize"
 #define SOUND_THEME_KEY "X-GNOME-Metatheme/SoundTheme"
 #define APPLICATION_FONT_KEY "X-GNOME-Metatheme/ApplicationFont"
 #define DESKTOP_FONT_KEY "X-GNOME-Metatheme/DesktopFont"
@@ -263,18 +272,8 @@ gnome_theme_read_meta_theme (GnomeVFSURI *meta_theme_uri)
   meta_theme_info->gtk_theme_name = g_strdup (str);
 
   str = gnome_desktop_item_get_string (meta_theme_ditem, GTK_COLOR_SCHEME_KEY);
-  if (str == NULL)
-  {
-    /* try to find the color scheme from the gtkrc */
-    gchar *gtkrc_file = gtkrc_find_named (meta_theme_info->gtk_theme_name);
-    if (gtkrc_file)
-    {
-      scheme = gtkrc_get_color_scheme (gtkrc_file);
-      g_free (gtkrc_file);
-    }
-    else
-      scheme = NULL;
-  }
+  if (str == NULL || !strcmp (str, ""))
+    scheme = gtkrc_get_color_scheme_for_theme (meta_theme_info->gtk_theme_name);
   else
     scheme = g_strdup (str);
 
@@ -302,6 +301,20 @@ gnome_theme_read_meta_theme (GnomeVFSURI *meta_theme_uri)
     }
   meta_theme_info->icon_theme_name = g_strdup (str);
 
+  str = gnome_desktop_item_get_string (meta_theme_ditem, CURSOR_THEME_KEY);
+  if (str != NULL) {
+    meta_theme_info->cursor_theme_name = g_strdup (str);
+
+    str = gnome_desktop_item_get_string (meta_theme_ditem, CURSOR_SIZE_KEY);
+    if (str)
+      meta_theme_info->cursor_size = (int) g_ascii_strtoll (str, NULL, 10);
+    else
+      meta_theme_info->cursor_size = 18;
+  } else {
+    meta_theme_info->cursor_theme_name = g_strdup ("default");
+    meta_theme_info->cursor_size = 18;
+  }
+
   str = gnome_desktop_item_get_string (meta_theme_ditem, APPLICATION_FONT_KEY);
   if (str != NULL)
     meta_theme_info->application_font = g_strdup (str);
@@ -330,6 +343,7 @@ read_icon_theme (GnomeVFSURI *icon_theme_uri)
   GnomeDesktopItem *icon_theme_ditem;
   gchar *icon_theme_file;
   const gchar *name;
+  const gchar *directories;
   const gchar *hidden_theme_icon;
 
   icon_theme_file = gnome_vfs_uri_to_string (icon_theme_uri, GNOME_VFS_URI_HIDE_NONE);
@@ -348,6 +362,15 @@ read_icon_theme (GnomeVFSURI *icon_theme_uri)
       return NULL;
     }
 
+  /* If index.theme has no Directories entry, it is only a cursor theme */
+  directories = gnome_desktop_item_get_string (icon_theme_ditem, "Icon Theme/Directories");
+  if (directories == NULL)
+    {
+      gnome_desktop_item_unref (icon_theme_ditem);
+      g_free (icon_theme_file);
+      return NULL;
+    }
+
   hidden_theme_icon = gnome_desktop_item_get_string (icon_theme_ditem, "Icon Theme/Hidden");
   if (hidden_theme_icon == NULL ||
       strcmp (hidden_theme_icon, "false") == 0)
@@ -937,112 +960,133 @@ add_common_theme_dir_monitor (GnomeVFSURI                *theme_dir_uri,
   return GNOME_VFS_OK;
 }
 
-
-static GdkPixbuf*
-gdk_pixbuf_from_xcursor_image(XcursorImage* cursor) {
-	GdkPixbuf* pixbuf;
+#ifdef HAVE_XCURSOR
+static GdkPixbuf *
+gdk_pixbuf_from_xcursor_image (XcursorImage *cursor) {
+  GdkPixbuf *pixbuf;
 #define BUF_SIZE sizeof(guint32) * cursor->width * cursor->height
-	guchar* buf = malloc(BUF_SIZE);
-	guchar* it;
-	memset(buf, '\0', BUF_SIZE);
+  guchar *buf = g_malloc0 (BUF_SIZE);
+  guchar *it;
 
-	for(it = buf; it < (buf + BUF_SIZE); it += 4) {
-		// can we get rid of this by using guint32 ?
+  for (it = buf; it < (buf + BUF_SIZE); it += 4) {
+    // can we get rid of this by using guint32 ?
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
-		// on little endianess it's BGRA to RGBA
-		it[0] = ((guchar*)(cursor->pixels))[it - buf + 2];
-		it[1] = ((guchar*)(cursor->pixels))[it - buf + 1];
-		it[2] = ((guchar*)(cursor->pixels))[it - buf + 0];
-		it[3] = ((guchar*)(cursor->pixels))[it - buf + 3];
+    // on little endianess it's BGRA to RGBA
+    it[0] = ((guchar *) (cursor->pixels))[it - buf + 2];
+    it[1] = ((guchar *) (cursor->pixels))[it - buf + 1];
+    it[2] = ((guchar *) (cursor->pixels))[it - buf + 0];
+    it[3] = ((guchar *) (cursor->pixels))[it - buf + 3];
 #else
-		// on big endianess it's ARGB to RGBA
-		it[0] = ((guchar*)cursor->pixels)[it - buf + 1];
-		it[1] = ((guchar*)cursor->pixels)[it - buf + 2];
-		it[2] = ((guchar*)cursor->pixels)[it - buf + 3];
-		it[3] = ((guchar*)cursor->pixels)[it - buf + 0];
+    // on big endianess it's ARGB to RGBA
+    it[0] = ((guchar *) cursor->pixels)[it - buf + 1];
+    it[1] = ((guchar *) cursor->pixels)[it - buf + 2];
+    it[2] = ((guchar *) cursor->pixels)[it - buf + 3];
+    it[3] = ((guchar *) cursor->pixels)[it - buf + 0];
 #endif
-	}
+  }
 
-	pixbuf = gdk_pixbuf_new_from_data((const guchar *)buf,
+  pixbuf = gdk_pixbuf_new_from_data ((const guchar *) buf,
 			GDK_COLORSPACE_RGB, TRUE, 8,
 			cursor->width, cursor->height,
 			cursor->width * 4,
-			(GdkPixbufDestroyNotify)g_free,
+			(GdkPixbufDestroyNotify) g_free,
 			NULL);
 
-	if(!pixbuf) {
-		g_free(buf);
-	}
+  if (!pixbuf)
+    g_free (buf);
 
-	return pixbuf;
+  return pixbuf;
 }
 
-
 static void
-look_for_cursor_theme (GnomeVFSURI *theme_dir_uri)
+read_cursor_theme (const gchar *theme_dir)
 {
-  GnomeVFSURI *cursors_dir_uri;
-  GnomeVFSFileInfo *file_info;
-  GnomeVFSResult result;
+  gchar *name;
+  gint i;
+  GArray *available_sizes;
+  XcursorImage *cursor;
+  GdkPixbuf *thumbnail = NULL;
+  GnomeThemeCursorInfo *cursor_theme_info;
+  const gint sizes[] = { 12, 16, 24, 32, 36, 40, 48, 64 };
+  const gint num_sizes = G_N_ELEMENTS (sizes);
 
-  cursors_dir_uri = gnome_vfs_uri_append_file_name (theme_dir_uri, "cursors");
-  file_info = gnome_vfs_file_info_new ();
-  result = gnome_vfs_get_file_info_uri (cursors_dir_uri, file_info, GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-  if (result == GNOME_VFS_OK && file_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
-  {
-    gchar *dir_name;
-    gchar *name;
-    gint i;
-    GArray *available_sizes;
-    XcursorImage *cursor;
-    GdkPixbuf *thumbnail = NULL;
-    GnomeThemeCursorInfo *cursor_theme_info;
-    gint sizes[] = { 12, 16, 24, 32, 36, 40, 48, 64, 0 };   
-    
-    dir_name = gnome_vfs_uri_to_string (theme_dir_uri, GNOME_VFS_URI_HIDE_NONE);
-    name = g_path_get_basename (dir_name);
+  name = g_path_get_basename (theme_dir);
 
-    available_sizes = g_array_sized_new (FALSE, FALSE, sizeof (gint), 8);
+  available_sizes = g_array_sized_new (FALSE, FALSE, sizeof (gint),
+				       num_sizes);
 
-    for (i = 0; sizes[i] != 0; i++)
-    {
-      cursor = XcursorLibraryLoadImage ("left_ptr", name, sizes[i]);
+  for (i = 0; i < num_sizes; i++) {
+    cursor = XcursorLibraryLoadImage ("left_ptr", name, sizes[i]);
 
-      if (cursor)
-      {
-        if (cursor->size != sizes[i])
-        {
-          XcursorImageDestroy (cursor);
-          cursor = NULL;
-        }
-        else
-        {
-          g_array_append_val (available_sizes, sizes[i]);
+    if (cursor) {
+      if (cursor->size == sizes[i]) {
+        g_array_append_val (available_sizes, sizes[i]);
 
-          if (thumbnail == NULL && i >= 1)
-            thumbnail = gdk_pixbuf_from_xcursor_image (cursor);
-        }
+        if (thumbnail == NULL && i >= 1)
+          thumbnail = gdk_pixbuf_from_xcursor_image (cursor);
       }
-    }
 
-    if (!thumbnail)
-    {
-      cursor = XcursorLibraryLoadImage ("left_ptr", name, g_array_index (available_sizes, gint, 0));
-      thumbnail = gdk_pixbuf_from_xcursor_image (cursor);
+      XcursorImageDestroy (cursor);
     }
-
-    cursor_theme_info = gnome_theme_cursor_info_new ();
-    cursor_theme_info->path = dir_name;
-    cursor_theme_info->name = name;
-    cursor_theme_info->sizes = available_sizes;
-    cursor_theme_info->thumbnail = thumbnail;
-    cursor_theme_info->priority = 0;
-    
-    g_hash_table_insert (cursor_theme_hash_by_uri, dir_name, cursor_theme_info);
-    add_data_to_hash_by_name (cursor_theme_hash_by_name, name, cursor_theme_info);
   }
+
+  if (!thumbnail && available_sizes->len != 0) {
+    cursor = XcursorLibraryLoadImage ("left_ptr", name,
+				      g_array_index (available_sizes, gint, 0));
+    if (cursor) {
+      thumbnail = gdk_pixbuf_from_xcursor_image (cursor);
+      XcursorImageDestroy (cursor);
+    }
+  }
+
+  cursor_theme_info = gnome_theme_cursor_info_new ();
+  cursor_theme_info->path = g_strdup (theme_dir);
+  cursor_theme_info->name = name;
+  cursor_theme_info->sizes = available_sizes;
+  cursor_theme_info->thumbnail = thumbnail;
+  cursor_theme_info->priority = 0;
+
+  if (!strcmp (name, "default")) {
+    cursor_theme_info->readable_name = g_strdup (_("Default Pointer"));
+  } else {
+    gchar *cursor_theme_file;
+    GnomeDesktopItem *cursor_theme_ditem;
+
+    cursor_theme_file = g_build_filename (theme_dir, "index.theme", NULL);
+    cursor_theme_ditem = gnome_desktop_item_new_from_file (cursor_theme_file, 0, NULL);
+    g_free (cursor_theme_file);
+
+    if (cursor_theme_ditem) {
+      const gchar *readable_name;
+
+      readable_name = gnome_desktop_item_get_string (cursor_theme_ditem, "Icon Theme/Name");
+
+      if (readable_name)
+        cursor_theme_info->readable_name = g_strdup (readable_name);
+      else
+        cursor_theme_info->readable_name = g_strdup (name);
+
+      gnome_desktop_item_unref (cursor_theme_ditem);
+    } else {
+      cursor_theme_info->readable_name = g_strdup (name);
+    }
+  }
+  g_hash_table_insert (cursor_theme_hash_by_uri, cursor_theme_info->path, cursor_theme_info);
+  add_data_to_hash_by_name (cursor_theme_hash_by_name, name, cursor_theme_info);
 }
 
+static void
+look_for_cursor_theme (const gchar *theme_dir)
+{
+  gchar *cursors_dir = g_build_filename (theme_dir, "cursors", NULL);
+
+  if (g_file_test (cursors_dir, G_FILE_TEST_IS_DIR) || g_str_has_suffix (theme_dir, "default"))
+    read_cursor_theme (theme_dir);
+
+  g_free (cursors_dir);
+}
+#endif /* HAVE_XCURSOR */
+
 static GnomeVFSResult
 add_common_icon_theme_dir_monitor (GnomeVFSURI                    *theme_dir_uri,
 				   gboolean                       *monitor_not_added,
@@ -1054,15 +1098,19 @@ add_common_icon_theme_dir_monitor (GnomeVFSURI                    *theme_dir_uri
   gboolean real_monitor_not_added = FALSE;
   GnomeVFSURI *index_uri;
 
+  uri_string = gnome_vfs_uri_to_string (theme_dir_uri,
+                                        GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+
+#ifdef HAVE_XCURSOR
   /* Look for cursor theme in the theme directory */
-  look_for_cursor_theme (theme_dir_uri);
+  look_for_cursor_theme (uri_string);
+#endif
 
   /* Add the handle for this directory */
   index_uri = gnome_vfs_uri_append_file_name (theme_dir_uri, "index.theme");
   update_icon_theme_index (index_uri, monitor_data->priority);
   gnome_vfs_uri_unref (index_uri);
 
-  uri_string = gnome_vfs_uri_to_string (theme_dir_uri, GNOME_VFS_URI_HIDE_NONE);
   result = gnome_vfs_monitor_add (& (monitor_data->common_icon_theme_dir_handle),
 				  uri_string,
 				  GNOME_VFS_MONITOR_DIRECTORY,
@@ -1356,12 +1404,19 @@ gnome_theme_info_find_by_type_helper (gpointer key,
 				      struct GnomeThemeInfoHashData *hash_data)
 {
   guint elements = GPOINTER_TO_INT (hash_data->user_data);
-  GnomeThemeInfo *theme_info = list->data;
 
-  if ((elements & GNOME_THEME_METACITY && theme_info->has_metacity) ||
-      (elements & GNOME_THEME_GTK_2 && theme_info->has_gtk) ||
-      (elements & GNOME_THEME_GTK_2_KEYBINDING && theme_info->has_keybinding))
-    hash_data->list = g_list_prepend (hash_data->list, theme_info);
+  do {
+    GnomeThemeInfo *theme_info = list->data;
+
+    if ((elements & GNOME_THEME_METACITY && theme_info->has_metacity) ||
+        (elements & GNOME_THEME_GTK_2 && theme_info->has_gtk) ||
+        (elements & GNOME_THEME_GTK_2_KEYBINDING && theme_info->has_keybinding)) {
+      hash_data->list = g_list_prepend (hash_data->list, theme_info);
+      return;
+    }
+
+    list = list->next;
+  } while (list);
 }
 
 GList *
@@ -1452,20 +1507,18 @@ gnome_theme_icon_info_compare (GnomeThemeIconInfo *a,
 GnomeThemeCursorInfo *
 gnome_theme_cursor_info_new (void)
 {
-  GnomeThemeCursorInfo *cursor_theme_info;
-
-  cursor_theme_info = g_new0 (GnomeThemeCursorInfo, 1);
-
-  return cursor_theme_info;
+  return g_new0 (GnomeThemeCursorInfo, 1);
 }
 
 void
 gnome_theme_cursor_info_free (GnomeThemeCursorInfo *cursor_theme_info)
 {
   g_free (cursor_theme_info->name);
+  g_free (cursor_theme_info->readable_name);
   g_free (cursor_theme_info->path);
   g_array_free (cursor_theme_info->sizes, TRUE);
-  g_object_unref (cursor_theme_info->thumbnail);
+  if (cursor_theme_info->thumbnail != NULL)
+    g_object_unref (cursor_theme_info->thumbnail);
   g_free (cursor_theme_info);
 }
 
@@ -1657,8 +1710,8 @@ gnome_theme_is_writable (const gpointer theme, GnomeThemeType type) {
       theme_path = ((const GnomeThemeIconInfo *) theme)->path;
       break;
     case GNOME_THEME_TYPE_CURSOR:
-      theme_path = ((const GnomeThemeIconInfo *) theme)->path;
-      break;      
+      theme_path = ((const GnomeThemeCursorInfo *) theme)->path;
+      break;
     case GNOME_THEME_TYPE_METATHEME:
       theme_path = ((const GnomeThemeMetaInfo *) theme)->path;
       break;
@@ -1699,6 +1752,186 @@ gnome_theme_info_register_theme_change (ThemeChangedCallback func,
   callbacks = g_list_prepend (callbacks, callback_data);
 }
 
+#ifndef HAVE_XCURSOR
+static gchar *
+read_current_cursor_font (void)
+{
+  DIR *dir;
+  gchar *dir_name;
+  struct dirent *file_dirent;
+
+  dir_name = g_build_filename (g_get_home_dir (), ".gnome2/share/cursor-fonts", NULL);
+  if (! g_file_test (dir_name, G_FILE_TEST_EXISTS)) {
+    g_free (dir_name);
+    return NULL;
+  }
+
+  dir = opendir (dir_name);
+
+  while ((file_dirent = readdir (dir)) != NULL) {
+    struct stat st;
+    gchar *link_name;
+
+    link_name = g_build_filename (dir_name, file_dirent->d_name, NULL);
+    if (lstat (link_name, &st)) {
+      g_free (link_name);
+      continue;
+    }
+
+    if (S_ISLNK (st.st_mode)) {
+      gint length;
+      gchar target[256];
+
+      length = readlink (link_name, target, 255);
+      if (length > 0) {
+        gchar *retval;
+        target[length] = '\0';
+        retval = g_strdup (target);
+        g_free (link_name);
+        closedir (dir);
+        return retval;
+      }
+
+    }
+    g_free (link_name);
+  }
+  g_free (dir_name);
+  closedir (dir);
+  return NULL;
+}
+
+static void
+read_cursor_fonts (void)
+{
+  gchar *cursor_font;
+  gint i;
+
+  const gchar *builtins[][4] = {
+    {
+      "gnome/cursor-fonts/cursor-normal.pcf",
+      N_("Default Pointer"),
+      N_("Default Pointer - Current"),
+      "mouse-cursor-normal.png"
+    }, {
+      "gnome/cursor-fonts/cursor-white.pcf",
+      N_("White Pointer"),
+      N_("White Pointer - Current"),
+      "mouse-cursor-white.png"
+    }, {
+      "gnome/cursor-fonts/cursor-large.pcf",
+      N_("Large Pointer"),
+      N_("Large Pointer - Current"),
+      "mouse-cursor-normal-large.png"
+    }, {
+      "gnome/cursor-fonts/cursor-large-white.pcf",
+      N_("Large White Pointer - Current"),
+      N_("Large White Pointer"),
+      "mouse-cursor-white-large.png"
+    }
+  };
+
+  cursor_font = read_current_cursor_font();
+
+  if (!cursor_font)
+    cursor_font = g_strdup (builtins[0][0]);
+
+  for (i = 0; i < G_N_ELEMENTS (builtins); i++) {
+    GnomeThemeCursorInfo *theme_info;
+    gchar *filename;
+
+    theme_info = gnome_theme_cursor_info_new ();
+
+    filename = g_build_filename (GNOMECC_DATA_DIR "/pixmaps", builtins[i][3], NULL);
+    theme_info->thumbnail = gdk_pixbuf_new_from_file (filename, NULL);
+    g_free (filename);
+
+    theme_info->name = g_build_filename (GNOMECC_DATA_DIR, builtins[i][0], NULL);
+    theme_info->path = g_strdup (theme_info->name);
+
+    if (!strcmp (theme_info->path, cursor_font))
+      theme_info->readable_name = g_strdup (builtins[i][2]);
+    else
+      theme_info->readable_name = g_strdup (builtins[i][1]);
+
+    g_hash_table_insert (cursor_theme_hash_by_uri, theme_info->path, theme_info);
+    add_data_to_hash_by_name (cursor_theme_hash_by_name, theme_info->name, theme_info);
+  }
+
+  g_free (cursor_font);
+}
+#endif /* !HAVE_XCURSOR */
+
+gboolean
+gnome_theme_color_scheme_parse (const gchar *scheme, GdkColor *colors)
+{
+  gchar **color_scheme_strings, **color_scheme_pair, *current_string;
+  gint i;
+
+  if (!scheme || !strcmp (scheme, ""))
+    return FALSE;
+
+  /* initialise the array */
+  for (i = 0; i < NUM_SYMBOLIC_COLORS; i++)
+    colors[i].red = colors[i].green = colors[i].blue = 0;
+
+  /* The color scheme string consists of name:color pairs, seperated by
+   * newlines, so first we split the string up by new line */
+
+  color_scheme_strings = g_strsplit (scheme, "\n", 0);
+
+  /* loop through the name:color pairs, and save the color if we recognise the name */
+  i = 0;
+  while ((current_string = color_scheme_strings[i++])) {
+    color_scheme_pair = g_strsplit (current_string, ":", 0);
+
+    if (color_scheme_pair[0] != NULL && color_scheme_pair[1] != NULL) {
+      g_strstrip (color_scheme_pair[0]);
+      g_strstrip (color_scheme_pair[1]);
+
+      if (!strcmp ("fg_color", color_scheme_pair[0]))
+        gdk_color_parse (color_scheme_pair[1], &colors[COLOR_FG]);
+      else if (!strcmp ("bg_color", color_scheme_pair[0]))
+        gdk_color_parse (color_scheme_pair[1], &colors[COLOR_BG]);
+      else if (!strcmp ("text_color", color_scheme_pair[0]))
+        gdk_color_parse (color_scheme_pair[1], &colors[COLOR_TEXT]);
+      else if (!strcmp ("base_color", color_scheme_pair[0]))
+        gdk_color_parse (color_scheme_pair[1], &colors[COLOR_BASE]);
+      else if (!strcmp ("selected_fg_color", color_scheme_pair[0]))
+        gdk_color_parse (color_scheme_pair[1], &colors[COLOR_SELECTED_FG]);
+      else if (!strcmp ("selected_bg_color", color_scheme_pair[0]))
+        gdk_color_parse (color_scheme_pair[1], &colors[COLOR_SELECTED_BG]);
+      else if (!strcmp ("tooltip_fg_color", color_scheme_pair[0]))
+        gdk_color_parse (color_scheme_pair[1], &colors[COLOR_TOOLTIP_FG]);
+      else if (!strcmp ("tooltip_bg_color", color_scheme_pair[0]))
+        gdk_color_parse (color_scheme_pair[1], &colors[COLOR_TOOLTIP_BG]);
+    }
+
+    g_strfreev (color_scheme_pair);
+  }
+
+  g_strfreev (color_scheme_strings);
+
+  return TRUE;
+}
+
+gboolean
+gnome_theme_color_scheme_equal (const gchar *s1, const gchar *s2)
+{
+  GdkColor c1[NUM_SYMBOLIC_COLORS], c2[NUM_SYMBOLIC_COLORS];
+  int i;
+
+  if (!gnome_theme_color_scheme_parse (s1, c1) ||
+      !gnome_theme_color_scheme_parse (s2, c2))
+    return FALSE;
+
+  for (i = 0; i < NUM_SYMBOLIC_COLORS; ++i) {
+    if (!gdk_color_equal (&c1[i], &c2[i]))
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
 void
 gnome_theme_init (gboolean *monitor_not_added)
 {
@@ -1717,8 +1950,8 @@ gnome_theme_init (gboolean *monitor_not_added)
   meta_theme_hash_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   icon_theme_hash_by_uri = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   icon_theme_hash_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-  cursor_theme_hash_by_uri = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-  cursor_theme_hash_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  cursor_theme_hash_by_uri = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
+  cursor_theme_hash_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
   theme_hash_by_uri = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   theme_hash_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
@@ -1775,6 +2008,11 @@ gnome_theme_init (gboolean *monitor_not_added)
   result = add_top_icon_theme_dir_monitor (top_theme_dir_uri, &real_monitor_not_added, 0, NULL);
   gnome_vfs_uri_unref (top_theme_dir_uri);
 
+#ifndef HAVE_XCURSOR
+  /* If we don't have Xcursor, use the built-in cursor fonts instead */
+  read_cursor_fonts ();
+#endif
+
   /* done */
   initted = TRUE;
   initting = FALSE;
